from __future__ import annotations

import datetime
import json
from contextvars import ContextVar
from typing import List, Any, Type

from pydantic_db_backend_common.utils import str_to_datetime_if_parseable


class Undefined:
    pass


class CustomJSONEncoder(json.JSONEncoder):
    def default(self, obj):
        match obj:
            case datetime.date() | datetime.datetime():
                return obj.isoformat()


class CustomJSONDecoder(json.JSONDecoder):
    def __init__(self, *args, **kwargs):
        super().__init__(object_hook=self.try_datetime, *args, **kwargs)

    @staticmethod
    def try_datetime(d: dict):
        ret = {}
        for key, value in d.items():
            match value:
                case str():
                    ret[key] = str_to_datetime_if_parseable(value)
                case _:
                    ret[key] = value
        return ret


def concat_aggregation(*aggregations: List[dict] | None) -> List[dict]:
    ret = []
    for agg in aggregations:
        if agg is not None and len(agg) != 0:
            ret.extend(agg)
    return ret


def resolve_undefined_type_context(
    context_var: ContextVar,
    new_type: Any,
    parameter: Any | None | Type[Undefined] = Undefined,
) -> Any:
    if parameter is Undefined:
        parameter = context_var.get()
    elif parameter is None:
        parameter = new_type()
    return parameter
